# ut.py
#
# Utility Functions to support PiGS application
#
# Copyright July 2020, Timothy Goihl
#
# return local time string and timezone minutes

import time
from datetime import timedelta, datetime

#https://docs.python.org/3.5/library/time.html

#Returns a LocalTime String 2020-07-15 23:11:27(CDT)
#and utc offset integer in minutes, like -300
#usage: timestr,offset = LocalTimeStr( datetime.utcnow() ) 
# doing my own local time convesion for my 'unaware' times

def LocalTimeStr(at, showSeconds=False, showDate=True, showTimeZone=True, WeekDay=True) :
    
    # fixed in 2.13 to display correct local-time
    t = time.localtime()
    tzstr = "(" + t.tm_zone + ")"
    tzmins = int(t.tm_gmtoff/60)
    localTime = at + timedelta(minutes=tzmins)
    
    dispFmt=''
    WkDyYr = "%a " if WeekDay else "%Y-"  # show Mon- or 2020- 
    if showDate : dispFmt += WkDyYr + '%m-%d '
    dispFmt += '%H:%M'
    if showSeconds : dispFmt += ':%S'
    
    lts = localTime.strftime(dispFmt)
    
    if showTimeZone : lts += tzstr
        
    return( lts, tzmins )


def VersionStringToFloat(vStr) :
    version = 0.0
    firstdp = -1
    
    for c in vStr :   # convert '5.4.1' into 5.41 for comparisons
        if c.isdigit() :
            version = version*10 + int(c)
            if firstdp > -1 : firstdp += 1  # count number of digits after first DP
        elif c == '.' and firstdp == -1 : firstdp = 0
    version = version / (10**firstdp)        

    return version

#Function to get total size of Airing list
#https://code.activestate.com/recipes/577504/

from sys import getsizeof, stderr
from itertools import chain
from collections import deque
try:
    from reprlib import repr
except ImportError:
    pass

def total_size(o, handlers={}, verbose=False):
    """ Returns the approximate memory footprint an object and all of its contents.

    Automatically finds the contents of the following builtin containers and
    their subclasses:  tuple, list, deque, dict, set and frozenset.
    To search other containers, add handlers to iterate over their contents:

        handlers = {SomeContainerClass: iter,
                    OtherContainerClass: OtherContainerClass.get_elements}

    """
    dict_handler = lambda d: chain.from_iterable(d.items())
    all_handlers = {tuple: iter,
                    list: iter,
                    deque: iter,
                    dict: dict_handler,
                    set: iter,
                    frozenset: iter,
                   }
    all_handlers.update(handlers)     # user handlers take precedence
    seen = set()                      # track which object id's have already been seen
    default_size = getsizeof(0)       # estimate sizeof object without __sizeof__

    def sizeof(o):
        if id(o) in seen:       # do not double count the same object
            return 0
        seen.add(id(o))
        s = getsizeof(o, default_size)

        if verbose:
            print(s, type(o), repr(o), file=stderr)

        for typ, handler in all_handlers.items():
            if isinstance(o, typ):
                s += sum(map(sizeof, handler(o)))
                break
        return s

    return sizeof(o)
