# Based off of howCode youtube video series "Make your own DNS servier in Python"
# https://www.youtube.com/playlist?list=PLBOh8f9FoHHhvO5e5HF_6mYvtZegobYX2
# https://github.com/howCodeORG/howDNS  Source code I started with, then butchered
#
# Also used https://pypi.org/project/pydnserver/ as a guide for the socket programming and timeouts
#  that project uses the dnspython library 
# https://stackoverflow.com/questions/17681230/how-make-dns-queries-in-dns-python-as-dig-with-additional-records-section
#
# details of the DNS Packet Structure
# http://www.tcpipguide.com/free/t_DNSMessageHeaderandQuestionSectionFormat.htm
# https://www2.cs.duke.edu/courses/fall16/compsci356/DNS/DNS-primer.pdf
#
# commands I found to see what process is using a port
#   sudo lsof -i :80
#   sudo lsof -i :53
#   sudo kill -9 pid 
#
# this was tested with dig
# sudo apt-get install dnsutils   (to get dig and nslookup)
# dig -h    for help
# +qr prints the outgoing DNS Query in detail
# +noedns removes the additional record with the cookie and other fields (dvr does not use EDNS)
#
# dig @192.168.1.200 microsoft.com +qr +noedns
# dig @192.168.1.200 -p 53593 microsoft.com  +qr +noedns


import socket, logging, json
from   datetime import datetime, timedelta
import time
import threading
from   ut import LocalTimeStr

# forward DNS queries I cant answer to gethostbyname
# Note that  gethostbyname_ex() looks in /etc/hosts for answers before going out on the internet 
from socket import gethostbyname     

dnsAbort = False
port = 53       # Port 53 is the 'system' port for DNS queries
#port = 53593   # port I used for testing with dig, or to test timeout
DNS_TIMEOUT = 300 # seconds.  300 default for 5 minutes, 30s for testing since dvr asks every 21s

SLOW_SEND_TIME = 25   # log message when total DNS response time is greater than 20s
SLOW_GHBN_TIME = 20  # log message when socket.GetHostByName() is slow (0.5s)

# 'dvr_hosts' is a dictionary of 'hosts' for the PiGS DNS Server
# This is a list of all DNS lookups performed by the DVR+
# epg and roproxy are the guide servers
# DVR+ uses DNS requests to microsoft.com,amazon.com,google.com and maybe echostar.com to determine
#   if it is connected to the internet.  If the DNS query succeeds, DVR+ displays 'Connected' on the Network page
# tr50 redirects to stbfw.echodata.tv cname and is for firmware updates
# PiGS DNS forwards urls not its 'hosts' list to googledns 8.8.8.8 and returns that result

# this hosts dictionary is url IP pairs
#   '*' means IP of PiGS, which will be filled in by this module

# hosts now comes from pigs.json, this is the format
# dvr_hosts = {
#    'epg.channelmastertv.com' : '*',        # fw 1.35 guide requests go here
#    'roproxy-guide.echostarcdn.com' : '*',  # fw 1.34 and before guide requests go here
#    'pool.ntp.org' : '162.159.200.123',     # time update server requests
#    'echostar.com'  :  '69.164.198.192',    # DVR+ uses these next 4 to determine if it is connected to the Internet
#    'microsoft.com' : '173.255.215.209',    # These are just some ntp server IPs addresses
#    'amazon.com'    : '104.236.116.147',    # DVR+ doesnt like it if I return IP's on the same network
#    'google.com'    : '138.236.128.112',    # DVR+ doesnt access these addresses, it just wants DNS to succeed.
#    'tr50.dishaccess.tv': '127.0.0.1',      # A tr50.dishaccess.tv CNAME stbfw.echodata.tv A 67.148.153.225
#    'stbfw.echodata.tv' : '127.0.0.1'       # firmware update
#}


def getflags(flags):

    byte1 = bytes(flags[:1])
    byte2 = bytes(flags[1:2])

    rflags = '' # return flags string of '1' and '0'
    
    QR = '1'   # Set the Response bit
    OPCODE = ''
    for bit in range(1,5):
        OPCODE += str(ord(byte1)&(1<<bit))
    AA = '0'   # Authority Answer '1'
    TC = '0'
    RD = '0'
    # Byte 2
    RA = '0'
    Z = '000'
    RCODE = '0000'

    # make binary strings, the convert to numbers, base 2
    bl = [int(QR+OPCODE+AA+TC+RD, 2), int(RA+Z+RCODE, 2)]
    ba = bytearray(bl)
    #bx = int(QR+OPCODE+AA+TC+RD, 2).to_bytes(1, byteorder='big')+int(RA+Z+RCODE, 2).to_bytes(1, byteorder='big')
    return ba

def getquestiondomain(data):

    state = 0            # state 0 >>> looking for length
    expectedlength = 0
    domainstring = ''
    domainparts = []     # a list like ['www','amazon','com','']
    x = 0  # char counter
    y = 0  # byte counter
    for byte in data:
        # this is buggy... byte==0 that breaks out of the loop is the zero of question, not the cnt of zero
        if state == 1:      # state 1 = reading x expectedlength chars
            if byte != 0:   # 
                domainstring += chr(byte)
            x += 1
            if x == expectedlength:
                domainparts.append(domainstring)
                domainstring = ''    # reset string
                state = 0            # state getLength byte
                x = 0                # reset char counter
            if byte == 0:
                domainparts.append(domainstring)
                break
        else:
            state = 1
            expectedlength = byte
        y += 1

    questiontype   = data[y:y+2]
    y += 2
    questionclass  = data[y:y+2]
    y += 2

    return (domainparts, questiontype, questionclass, y)


def buildquestion(domainname, rectype):
    qbytes = b''

    for part in domainname:
        length = len(part)
        qbytes += bytes([length])

        for char in part:
            qbytes += ord(char).to_bytes(1, byteorder='big')

    if rectype == 'a':
        qbytes += (1).to_bytes(2, byteorder='big')

    qbytes += (1).to_bytes(2, byteorder='big')

    return qbytes

def rectobytes(domainname, rectype, recttl, recval):

    # pointer to name in the Question - it's at offset 12 
    rbytes = b'\xc0\x0c'

    if rectype == 'a':
        rbytes = rbytes + bytes([0]) + bytes([1])

    # Class type 1 INternet
    rbytes = rbytes + bytes([0]) + bytes([1])

    rbytes += int(recttl).to_bytes(4, byteorder='big')

    # type A record get 4 bytes of IP address
    if rectype == 'a':
        rbytes = rbytes + bytes([0]) + bytes([4])

        for part in recval.split('.'):
            rbytes += bytes([int(part)])
    return rbytes

def buildresponse(data, pigsIP, addr, hosts):
    global LastDnsError
    
    # Get Transaction ID
    TransactionID = data[:2]

    # Get the flags, set response.  RCODE is the lowest 4-bits so we can just add it in
    Flags = getflags(data[2:4])

    # Question Count
    QDCOUNT = b'\x00\x01'

    # extract requested domain, check my hosts file
    domainname, questiontype, questionclass, msglen = getquestiondomain(data[12:])
    msglen += 12
    
    #build a domain name string from domain tuple
    qhost = '.'.join(domainname)[0:-1]  # strip off trailing '.'
    qhost = qhost.lower()
    
    # After DNS packet has been decoded, we know its length
    # Check if we grabbed more data than we processed, meaning a dns msg might be skipped
    # This is to characterize this module.  
    # The error will be viewable on the /pigstatus page
    # Dig sends some extra bytes...   which causes this to print during testing...
    
    #if len(data) != msglen :
    #    s = 'Msg length error. DnsMsgLen({}) SocketLen({}) {}'.format(msglen, len(data), qhost)
    #    logging.warning("PIGSDNS: %s", s )
    #    LastDnsError = s
    
    logging.debug("PIGSDNS: DNS request(%s) from %s for %s", hex(int.from_bytes(TransactionID[:2],byteorder ='big')), addr, qhost )
    
    # Create DNS body
    dnsbody = b''   # empty binary string
    rectype = 'a'
    
    # Get answer for query, Check PiGS 'hosts file' domains first, return PiGS IP if found
    
    # The domain name is the dictionary key, see if it exists
    if qhost in hosts :
        ANCOUNT = (1).to_bytes(2, byteorder='big')    # if found, return 1 record
        hostip  = hosts[qhost]  # lookup host IP
        if hostip == '*' :      # '*' means use IP discovered by PiGS
            hostip = pigsIP
        logging.debug("PIGSDNS: %s resolved to %s", qhost, hostip )
        dnsbody = rectobytes(domainname, rectype, 0, hostip)  # 0 = TTL, do not cache my response
        
    else :   # forward anything else to the system DNS, Single IP request method.
             # amazon.com, microsoftcom, etc.

        # DVR doesnt actually use these forwarded IP's it just want a DNS response to find out if it is online
        # cant receive any other DNS requests until this function returns.  what if it is ?
        
        acnt = 0  # if error, will return 0 answer records
        try :
            hostip   = gethostbyname(qhost)
            dnsbody += rectobytes(domainname, rectype, 0, hostip )  # 0 = TTL, do not cache my response
            acnt     = 1  # if success, ghbn returns 1 IP address
            
        except Exception as inst:
            Flags[1] += 2   # signal a dns server error for all errors
            #Flags[1] += 3   # signal a dns name error for all errors
            # throws socket.gaierror error on bad
            d = datetime.utcnow() - lastDnsRequestTime  # how long did this take?
            fd = d / timedelta(seconds=1)  # convert to floating secs
            logging.debug("PIGSDNS:GHBN Error(%s) %s secs %s: %s", qhost, fd, addr, inst )
            
        d = datetime.utcnow() - lastDnsRequestTime
        fd = d / timedelta(seconds=1)  # convert to floating sec
        if fd >= SLOW_GHBN_TIME :  # 0.5s
            logging.debug("PIGSDNS:GHBN Info(%s) %s secs %s: ", qhost, fd, addr )
        
        ANCOUNT = (acnt).to_bytes(2, byteorder='big')

    #logging.info("PIGSDNS: From %s %s resolved to %s", addr, qhost, hostip )

    # Nameserver, Additonal Count
    NSCOUNT = (0).to_bytes(2, byteorder='big')
    ARCOUNT = (0).to_bytes(2, byteorder='big')

    # Build an Answer Header
    dnsheader = TransactionID+Flags+QDCOUNT+ANCOUNT+NSCOUNT+ARCOUNT

    # Repeat the Question.  domainname is a tuple, rectype is 'a'
    dnsquestion = buildquestion(domainname, rectype)

    return dnsheader + dnsquestion + dnsbody, qhost


PigsDnsThread   = threading.Thread()
LastDnsError    = 'None.'

def PigsDnsStart(listenIP, IPaddr, hostsd) :
    global dnsAbort
    global PigsDnsThread
    global lastDnsRequestTime
    global LastDnsError
    
    LastDnsError    = 'None.'
    lastDnsRequestTime = None
    
    PigsDnsThread = threading.Thread(target=PigsDNSserverRun, name='DNS-Server', daemon=True, args=(listenIP, IPaddr, hostsd,) )
    PigsDnsThread.start()

    logging.info("PIGSDNS: DnsStart ListenIP=%s, DefaultIP=%s, Alive:%s", listenIP, IPaddr, PigsDnsThread.is_alive())

    return PigsDnsThread

def PigsDnsStop(message=None) :
    global dnsAbort
    global PigsDnsThread
    global lastDnsRequestTime
    global LastDnsError
    
    LastDnsError = message if message else 'None.'  # Display message 
    lastDnsRequestTime = None
    dnsAbort = True
    return PigsDnsThread

def PigsDnsAlive() :
    global PigsDnsThread    
    return PigsDnsThread.is_alive()

def PigsDnsStatus() :
    global lastDnsRequestTime
    global PigsDnsThread
    global LastDnsError
    dnsdict = {}
    dnsdict['Status'] = 'Enabled' if PigsDnsThread.is_alive() else 'Disabled'
    if lastDnsRequestTime :
        lrt,lz = LocalTimeStr(lastDnsRequestTime, showSeconds=True, showDate=False, showTimeZone=False, WeekDay=True)
    else :
        lrt = "Never."
    dnsdict['DvrLastSeen'] = lrt
    dnsdict['LastErr'] = LastDnsError
    
    #return PigsDnsThread.is_alive(), lastDnsRequestTime
    return dnsdict

#    logging.debug("PIGSDNS: DNS request(%s) from %s for %s", hex(int.from_bytes(TransactionID[:2],byteorder ='big')), addr, qhost )


#https://stackoverflow.com/questions/8748711/udp-recv-recvfrom-multiple-senders/8748884#8748884
#https://stackoverflow.com/questions/26184970/do-udp-sockets-delineate-received-data-into-distinct-messages-or-does-recv-read/26185032#26185032
#https://en.wikipedia.org/wiki/Ethernet_frame

# This is the 'run' or start method
requestCounter = 0
lastDnsRequestTime = None

def PigsDNSserverRun( listen_ip, pigsIP, dnshosts) :
    global dnsAbort
    global requestCounter
    global lastDnsRequestTime
    global LastDnsError

    timeoutSecs = 1        # DNS will 'timeout' every 1 sec. we monitor if requests are coming in for /pigstatus
    timeoutCounter = 0
    
    dnsAbort = False       # check abort flag on each timeout
    
    # set up the IP addr dns server will return to DVR
    respIP = pigsIP
    if listen_ip != '0.0.0.0' :  # if dns is going to listen to a specific IP, that will be the default response IP
        respIP = listen_ip
    
    #logging.info("PIGSDNS: Default IP %s", respIP )

    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.settimeout(timeoutSecs)  # recvfrom() will timeout every 1 second, ensures we can abort thread
    
    try :
        sock.bind((listen_ip, port))
        logging.info("PIGSDNS: Waiting for DNS requests on port %s", port )
    except:
        logging.warning("PIGSDNS: Could not bind to port %s", port )
        logging.warning("PIGSDNS: On Linux, dnsmasq loaded?  try 'sudo lsof -i :53' for pid" )
        logging.warning("PIGSDNS: On Windows, check firewall - allow PiGS full access" )
        dnsAbort = True

    while not dnsAbort:
        
        try :    # recvfrom will receive 1 and only 1 udp message per call
            
            data, addr = sock.recvfrom(512)  # DNS UDP messages limited to 512 bytes

        except socket.timeout:  # periodic timeout is expected, will use to monitor DVR requests
            timeoutCounter += 1   # count timeouts, warn every 5 mins if DVR's not making DNS requests
            if timeoutCounter >= DNS_TIMEOUT/timeoutSecs :
                if requestCounter == 0 :
                    dnsmsg = "No DVR DNS request for {} seconds...".format(timeoutCounter*timeoutSecs)
                    logging.warning("PIGSDNS: %s", dnsmsg )
                    LastDnsError = dnsmsg
                requestCounter = 0;
                timeoutCounter = 0
            continue  # to wait for next request
        
        except Exception as serr:
            logging.error(u'PIGSDNS: DNS Socket recvfrom() Error: {err}'.format(err=serr))
            continue  #  to wait for next request
            
        else :  # if no exception, process the Request
            lastDnsRequestTime = datetime.utcnow()  # time of this request
            requestCounter += 1
            LastDnsError = 'None.'   # reset error message when we get a request

            r, host = buildresponse(data, respIP, addr, dnshosts)

        try :  # to send the response back to the host as a UDP message
            
            sock.sendto(r, addr)
            #    logging.info("PIGSDNS: DNS skipping(%s) from %s", hex(int.from_bytes(r[:2],byteorder ='big')), addr)
            
        except Exception as serr:
            logging.error(u'PIGSDNS: DNS Socket sendto() Error: {err}'.format(err=serr))

        # log very long requests,  lastDnsRequestTime = datetime.utcnow()
        d  = datetime.utcnow() - lastDnsRequestTime
        fd = d / timedelta(seconds=1)  # convert to floating sec
        if fd >= SLOW_SEND_TIME :  # 20s
            logging.info("PIGSDNS:Slow DNS Response (%s) %s Secs %s: ", host, fd, addr )

    logging.info("PIGSDNS: Stopping DNS & closing socket.")
    
    try :
        sock.shutdown(socket.SHUT_RDWR)
    except :
        pass
    try :
        sock.close()
    except :
        pass
